@@ -4,27 +4,14 @@ use 5.008000;
 use strict;
 use warnings;
 
-our $VERSION = '0.04';
+our $VERSION = '0.06';
 
 use AnyEvent;
 use Carp qw( croak );
 
-my %REGISTERED_EVENTS;
+my %REGISTERED_HANDLERS;
 my %MODULES_IDX;
 
-__PACKAGE__->register( __PACKAGE__,
-  'finalize:r' => sub {
-    my $cb = pop if ref( $_[-1] ) eq 'CODE';
-
-    undef %REGISTERED_EVENTS;
-    undef %MODULES_IDX;
-
-    if ( defined $cb ) {
-      AE::postpone { $cb->() };
-    }
-  }
-);
-
 
 sub register {
   my $class        = shift;
@@ -42,16 +29,16 @@ sub register {
 
   while ( my ( $event_name, $handler ) = each %handlers ) {
     next if exists $events_idx->{$event_name};
-    $events_idx->{$event_name} = 1;
 
-    unless ( exists $REGISTERED_EVENTS{$event_name} ) {
-      $REGISTERED_EVENTS{$event_name} = [];
+    $events_idx->{$event_name} = 1;
+    unless ( exists $REGISTERED_HANDLERS{$event_name} ) {
+      $REGISTERED_HANDLERS{$event_name} = [];
     }
     if ( $event_name =~ m/\:r$/ ) {
-      unshift( @{ $REGISTERED_EVENTS{$event_name} }, $handler );
+      unshift( @{ $REGISTERED_HANDLERS{$event_name} }, $handler );
     }
     else {
-      push( @{ $REGISTERED_EVENTS{$event_name} }, $handler );
+      push( @{ $REGISTERED_HANDLERS{$event_name} }, $handler );
     }
   }
 
@@ -61,44 +48,44 @@ sub register {
 sub send_event {
   my $class      = shift;
   my $event_name = shift;
+  my $cb         = pop if ref( $_[-1] ) eq 'CODE';
 
   unless ( defined $event_name ) {
     croak 'Event name must be specified';
   }
 
-  return unless exists $REGISTERED_EVENTS{$event_name};
+  return unless exists $REGISTERED_HANDLERS{$event_name};
 
-  my @handlers = @{ $REGISTERED_EVENTS{$event_name} };
+  my @handlers = @{ $REGISTERED_HANDLERS{$event_name} };
 
-  if ( ref( $_[-1] ) eq 'CODE' ) {
-    my $cb = pop @_;
-    $class->_async_call_handler( \@handlers, [@_], $cb );
+  if ( defined $cb ) {
+    $class->_process_async( \@handlers, [@_], $cb );
+    return;
   }
-  else {
-    foreach my $handler (@handlers) {
-      $handler->(@_);
-    }
+
+  foreach my $handler (@handlers) {
+    $handler->(@_);
   }
 
   return;
 }
 
-sub _async_call_handler {
-  my $class = shift;
-  my $handlers   = shift;
-  my $args       = shift;
-  my $cb         = shift;
+sub _process_async {
+  my $class    = shift;
+  my $handlers = shift;
+  my $args     = shift;
+  my $cb       = shift;
 
   my $handler = shift @{$handlers};
 
-  unless ( defined $handler ) {
-    $cb->();
-    return;
-  };
-
   $handler->( @{$args},
     sub {
-      $class->_async_call_handler( $handlers, $args, $cb );
+      if ( @{$handlers} ) {
+        $class->_process_async( $handlers, $args, $cb );
+        return;
+      }
+
+      $cb->();
     }
   );
 
